#include "combine.h"
#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>

/* Argument Calculation Program
 *
 * Description: Reads the arguments and also checks for thir valdiity.
 *
 */

int processArgs(int argc, char **argv, struct args *args, int rank) {
  int i,j,k;
  int done;
  int error = FALSE;
  
  char tmpstring[MAXCHAR], numstring1[MAXCHAR], numstring2[MAXCHAR], 
    numstring3[MAXCHAR];

  if (argc < 3) return 1;
    
  // Define default values for arguments

  args->numelem = 1000;
  args->type = -1;
  args->numiter = 10;
  args->stepsize = 0;
  args->numsteps = 1;
  args->print = FALSE;
  args->output = FALSE;
  args->compute = FALSE;
  args->notpow = FALSE;
  args->block = FALSE;
  args->testrank = 0;
  args->check = FALSE;
  args->ring = TRUE;
  args->printtype = 1;
  
  for (k=1; k < (argc-2); k++) {
    strcpy(tmpstring,argv[k]);

    // read the '-'
    if (tmpstring[0] == '-') {
      numstring1[strlen(tmpstring)-2] = '\0';
      // check the argument type
      switch(tmpstring[1]) {     
	// Number of elements
        case 'e': {
          for(i=2;i<strlen(tmpstring);i++) {
            if (isdigit(tmpstring[i])) {
              numstring1[i-2] = tmpstring[i];
            } else {
	      // If number of arguments is alphabet give an error
              error=TRUE;
              
            }
          }
	  // Set the number of elements
          args->numelem = atoi(numstring1);
          break;
        }
	  // Number of iterations
      case 'i': {
        for(i=2;i<strlen(tmpstring);i++) {
          if (isdigit(tmpstring[i])) {
            numstring1[i-2] = tmpstring[i];
          } else {
	    // If number of arguments is alphabet give an error
            error=TRUE;
          }
        }
	// Set the number of iterations
        numstring1[strlen(tmpstring)] = '\0';
        args->numiter = atoi(numstring1);
        break;
      }

	// Increasing steps
      case 't': {
        done = FALSE;
        i = 2;
	// First get the size of element increase
        while ((i<strlen(tmpstring)) && (!done)) {
            if (isdigit(tmpstring[i])) {
              numstring1[i-2] = tmpstring[i];
            } else {
              numstring1[i-2] = '\0';
	      // Next get the number of steps
              if (tmpstring[i] == 'x') {
                for (j=(i+1);j<strlen(tmpstring);j++){
                  if (isdigit(tmpstring[j])) {
                    numstring2[j-(i+1)] = tmpstring[j];
                  }
                }
                numstring2[j-i] = '\0';
              } else {
                error=TRUE;
                done = FALSE;
              }
            }
            i++;
          }
	// Set the step size and number of steps
          args->stepsize = atoi(numstring1);
          args->numsteps = atoi(numstring2);
          break;
      }
	// Non-verbose print (for plotting the results)
	// 0 is prints minimum time, 1 average, 2 maximum. Default is 1.
      case 'p': {
        args->print = TRUE;
        if (strlen(tmpstring) == 3) {
          if (isdigit(tmpstring[2])) {
            numstring1[0] = tmpstring[2];
            numstring1[1] = '\0';
          } else {
            error=TRUE;
          }
        } 

        if (strlen(tmpstring) > 3) error = TRUE;
        args->printtype = atoi(numstring1);
        if ((args->printtype < 0) || (args->printtype > 2)) error = TRUE;
        break;
      }
	// Output the results together with time (used for debugging).
	// The number next to o prints the results for the corresponding node.
	// Default is 0.
      case 'o': {
        args->output = TRUE;
        if (strlen(tmpstring) > 1) {
          i = 2;
          done = FALSE;
          while ((i<strlen(tmpstring)) && !done) {
            if (isdigit(tmpstring[i])) {
              numstring3[i-2] = tmpstring[i];
            } else {
              error = TRUE;
              done = TRUE;
            }
            i++;
          }
	  // Set the node to print
          args->testrank = atoi(numstring3);
        } else {
	  // Else use default 0
          args->testrank = 0;
        }
        
        break;
      }
	// Obsolete (intersperse with computation)
      case 'c': {
        args->compute = TRUE;
        break;
      }
	// Uses values of 10 for input instead of default powers of 2
      case 'r': {
        args->notpow = TRUE;
        break;
      }
	// Obsolete (use blocking sends-recieves)
      case 'b': {
        args->block = TRUE;
        break;
      }
	// Verify results (see verify.c)
      case 'v': {
        args->check = TRUE;
        break;
      }
      default: {
        error=TRUE;
        
      }
      }
    } else {
      error=TRUE;
        
    }    
  }//end of optional arguments

  // Calculate the type of algorithm to use

  strcpy(tmpstring, argv[argc-2]);
  if (strcmp(tmpstring,"ring") == 0) {
    args->type = 0;
  } else if (strcmp(tmpstring,"fan") == 0) {
    args->type = 1;
  } else if (strcmp(tmpstring,"bidir") == 0) {
    args->type = 2;
  } else if (strcmp(tmpstring, "rec") == 0) {
    args->type = 3;
  } else if (strcmp(tmpstring, "buck") == 0) {
    args->type = 4;
  } else if (strcmp(tmpstring, "mpi") == 0) {
    args->type = 5;
  } else if (strcmp(tmpstring, "pipe") == 0) {
    args->type = 6;
  } else if (strcmp(tmpstring, "pipe2") == 0) {
    args->type = 6;
    args->ring = FALSE;
  } else if (strcmp(tmpstring, "bing") == 0) {
    args->type = 7;
  } else if (strcmp(tmpstring, "naive") == 0) {
    args->type = 8;
  } else if (strcmp(tmpstring, "fourway") == 0) {
    args->type = 9;
  } else {
    error=TRUE;    
  }
  if (!(args->print) && (!rank)) printf("\n");

  // Calculate the type of operation
  strcpy(tmpstring, argv[argc-1]);
  if (strcmp(tmpstring,"gather") == 0) {
    args->op = 0;
  } else if (strcmp(tmpstring,"reduce") == 0) {
    args->op = 1;
    if (args->type == 7) error = TRUE;
  } else if (strcmp(tmpstring,"redscat") == 0) {
    args->op = 2;
  } else {
    error = TRUE;
  }

  if (error) {
    return 1;
  }

  return 0;
}

// If error print out error message
void argErr() {
  printf(
"Error in Input!\n\
\n\
Format: gather [OPTIONS...] TYPE OPERATION\n\
OPTIONS:\n\
-e#: Number of Elements\n\
-i#: Number of Iterations\n\
-t#x#: Use Time Steps, Size of Increment, Number of Increments\n\
-p[#]: Create a printable format, 0=mintime, 1=avgtime, 2=maxtime\n\
-o[#]: Print results from node # (default 0)\n\
-c: Intersperse with computation (obsolete)\n\
-r: Use values of 10 as input elements\n\
-b: Use blocking send/recv (obsolete)\n\
-v: Verify routine 
\n\
TYPE:\n\
ring: Ring\n\
pipe: Ring Pipelined\n\
pipe2: Tree Pipelined\n\
fan: Fan-in/Fan-out\n\
bidir: Bidirectional Exchange\n\
rec: Recursive Halving\n\
buck: Bucket\n\
mpi: MPI_Reduceall\n\
bing: Full Fan-in\n\
naive: Full Fan-out\n\
fourway: Repeated 4-way Broadcast\n\
\n\
OPERATION:\n\
gather: All-Gather\n\
reduce: All-Reduce\n\
redscat: Reduce-Scatter\n\
\n");  
}  





